#!/usr/bin/env Rscript 
#chmod +x

# user case OpenMOLE : but : tracer la courbe de régression de l'aire en fonction de T, pour différents tau, à partir d'un
# fichier csv produit dans OpenMole.

########### Library  ##############
library(akima) # pour interp
library(RColorBrewer)
library(stringr)

########################
###    A remplir     ###
########################
n= 50  # nb de répétition (nb de colonne dans la future matrice)

# le nom du fichier qui contient les résultats de l'exploration
nameFile = "resultatExploration.csv" 
resExplo = read.csv(nameFile,header=TRUE)

# fin de "à replir"

########### create vector  ##############   
resSizeI = resExplo[,"TailleIni"]
resAreaF = resExplo[,"AireFinale"]  
tau = resExplo[,"tau"]
t = resExplo[,"T"]   
initialPopSize = resExplo[,"initialPopSize"] 
proportionMowing = resExplo[,"proportionMowing"]  

# les valeurs des paramètres (choisies dans openMole, mais on les retrouve ici)
# Remarque : on trie les valeurs par ordre croissant
valProportionMowing = sort(proportionMowing[-which(duplicated(proportionMowing))])  
valTau = sort(tau[-which(duplicated(tau))])  
valT = sort(t[-which(duplicated(t))])  
valInitialPopSize = sort(initialPopSize[-which(duplicated(initialPopSize))])  

# la longeur des vecteurs de paramètres
lgProportionMowing = length(valProportionMowing)
lgTau = length(valTau)
lgT=length(valT)
lgInitialPopSize = length(valInitialPopSize)


################################
###     FONCTIONS UTILES     ###
################################
source("createMatrix.R")


##################################
###    CREATION DES MATRICES   ###
##################################
# création d'une matrice (une ligne correspondont aux répétitions d'une même simulation ie mêmes paramètres)
# les 4 premières colonnes sont les params de gestion, dans les suivantes la valeurs de l'aire pour ce jeu de params.
# Si pas assez de valeur (simus en cours), on met NA (à la fin de la ligne)
# requiert les vecteurs créés plus haut : valTruc et lgTruc
resAreaF2 = createMatrix(resAreaF)
resSizeI2 = createMatrix(resSizeI)


########################################
###     NETTOYAGES ECHEC POP INI     ###
########################################
# on met la veleur NA dans les matrices finales, là ou on n'a pas réussi à créer la pop initiale (taille 0)

selNoInitialPop = which(resSizeI2[, 5:(n+4)] < 1500)
length(selNoInitialPop)  # le nombre de pop ini qu'on n'a pas créé, pour cause de pas de chance
resAreaF2[, 5:(n+4)][selNoInitialPop] = NA


#########################################
###  TOBBIT : FABRIQUE DES MATRICES   ###
#########################################
# tobbit (estimation de moyenne et variance pour si l'échantillon a une loi normale), 
# donne l'info du nb d'extinction, de la moyenne et la variance. La fonction agit sur une ligne.

quantiteTobit = function(x){
  y = x[!is.na(x)]
  nbExtinction = sum(y==0)
  
  if(nbExtinction == 0){ matrix(c(nbExtinction, mean(y), sqrt(sd(y))), ncol = 3) }
  else if (nbExtinction < (n/2) ){  # choix à faire pour la comparaison
    negLogLik <- function(theta, obs = y) {
      prob_censored <- pnorm(0, mean = theta[1], sd = exp(theta[2]), log = TRUE)
      - sum(ifelse(obs > 0, dnorm(obs, theta[1], exp(theta[2]), log = TRUE), prob_censored))
    }
    opt <- optim( c(0, 0), negLogLik)
    meanTobit = opt$par[1]
    varTobit = exp(opt$par[2])
    matrix(c(nbExtinction, meanTobit, varTobit),ncol = 3) }
  else{ matrix(c(nbExtinction, 0,0),ncol = 3) }
}


# aire moyenne Finale
resMoyAreaFTobbit = t(apply(resAreaF2[,5:(n+4)], 1, quantiteTobit))
temp = cbind(resAreaF2[,"ProportionMowing"], resAreaF2[,"Tau"], resAreaF2[,"InitialPopSize"], resAreaF2[,"T"], resMoyAreaFTobbit)
colnames(temp) = c ("ProportionMowing","Tau","InitialPopSize","T","NbExt","moyTobbit","varTobbit")
resMoyAreaFTobbit = temp


# fonction qui trace l'aire myenne finale, pour un jeu de paramètre de gestion donné
plotAireVsT = function(TautoPlot, InitialPopSizetoPlot = 1500, ProportionMowingtoPlot =0.9){
  
  sel = which(resMoyAreaFTobbit[, "ProportionMowing"] == ProportionMowingtoPlot & 
                resMoyAreaFTobbit[, "Tau"]   == TautoPlot &
                resMoyAreaFTobbit[, "InitialPopSize"]  == InitialPopSizetoPlot )
  
  xToplot = resMoyAreaFTobbit[sel,"T"]  #toPlot
  valPlot = resMoyAreaFTobbit[sel,6]  # 6 pour la moyenne de tobbit, 5 our le nb d'extinction, 7 pour la variance
  
  # intermède pour préparer l'extinction
  tempExt = temp[sel,5]
  nbExtTot = sum(tempExt)
  nbExtSansMult = length(temp[-which(temp == 0)])
  # fin intermède
  
  plot(xToplot, valPlot, 
       xlab = "T" , xlim = range(valT), 
       ylab = "Aire Moyenne Finale", sub = paste("Nombre d'extinction = ", nbExtTot, " (sur ", n, "x", lgT, " simulations)",sep = ""))
  title( main = "Aire Moyenne Finale VS T")
  title(main = paste("\n", "\n", "ProportionMowing = ", ProportionMowingtoPlot, ", Tau = ", TautoPlot, 
                     ", InitialPopSize = ",InitialPopSizetoPlot, sep = ""),
        cex.main=1.3, font.main=1)
  
  
  # reprise intermède, tracer les endroits d'extinction  
  col = brewer.pal(6,"Blues")
  # trouver le premier indice d'extinction:
  idFirstExt = if(sum(tempExt >= 1)!=0){ which(tempExt >= 1 )[1] } else{max(length(xToplot))+1}
  abline(v = xToplot[idFirstExt], lwd=1, col = col[4])
  # trouver le premier indice ou plus de 50 % d'extinction:
  idPercentExt = if(sum(tempExt/n >= 0.5)!=0){ which(tempExt/n >= 0.5 )[1] } else{length(xToplot)+1}
  abline(v = xToplot[idPercentExt], lwd=1, col = col[5])
  # fin inrermède
  
  # on ne fait pas que ploter, on renvoit aussi des trucs
  return(list(xToplot, valPlot))
}

# exemple
TautoPlot = 10
InitialPopSizetoPlot = 1500
ProportionMowingtoPlot = 0.9
plotAireVsT(TautoPlot,InitialPopSizetoPlot,ProportionMowingtoPlot)



############################################
###    Régression : Aire moyenne vs T    ###  
###    +  variation de  tau              ###
############################################

InitialPopSizetoPlot = 1500 ; ProportionMowingtoPlot = 0.9

# on enregistre les coeffs de la régression (sur T), pour chaque tau dans une liste
# le val max sert pour la fenêtre graphique
maxVal = 0
resCoeff = NULL
for(i in 1:lgTau){
  TautoPlot = valTau[i]
  
  res = plotAireVsT(TautoPlot)
  
  x1 = res[[1]] ; x1
  y1 = res[[2]] ; y1
  
  # on ajoute un seuil
  seuil = 0
  x1 = x1[which(y1> seuil)] 
  y1 = y1[which(y1> seuil)]
    
  model = lm( y1 ~  I(x1) + I(x1^2)  )
  resCoeff[[i]] = model$coefficients
  tempEstimY = fitted(model)
  maxVal = max(c(maxVal,tempEstimY))
  myPredict <- predict( model ) 
  lines(x1, myPredict, col=2, lwd=2 )  
  text(0, min(y1), adj = c(0,0), paste("R² = ", round(summary(model)$r.squared,4)) )
  
}

resCoeff


# ATTENTION VALIDE UNIQUEMENT SI SUITE CONTINUE DE PUISSANCE DEPUIS 0 : 1, X, X^2,... sinon À ADAPTER !
# fonction pour tracer la courbe de régression à partir des coeffs 
evaluationRegression = function(coeff,t){
  l = length(coeff)
  temp1 = rep(t,l)
  temp2 = 0:(l-1)
  res = sum(coeff*(temp1^temp2))
  res
}


# choix de couleurs
col = colorRampPalette(brewer.pal(9,"YlOrRd")[4:9])(lgTau) 

# on plot les courbes de régression sur T, pour chaque valeur de tau
x = seq(0,20,0.2)

# pour enregistrer l'image
#pdf("aire moyenne vs T.pdf")
png("aire moyenne vs T.png")

# fenetre graphique
plot(1, type = "n", xlim = c(0,20), ylim = c(0,maxVal), xlab = "T", ylab = "mean final area")
# trace les courbes de regressions obtenues
for(i in 1:lgTau){
  temp = sapply(x, function(t){evaluationRegression(resCoeff[[i]],t)})
  lines(x,temp, col = col[i])
}
title(main = "Regression curves of the area as a function of T")
selLegende = floor(seq(1,lgTau,length.out = 5))
legend("bottomleft",col = col[selLegende] , paste("tau =", valTau[selLegende]) , pch = c(1,1))

dev.off() 




